/** Store information about the ball.
 * The information included, the placement of the ball, current velocity and direction, 
 * aswell as its size, point value and horisontal movement
 */


public class Ball {
	

	
	private int ballSize = 0;
	private int ballPxlSize = 0;
	private int ballBounceSize;
	private int ballPointValue;
	
	private int x = 0;
	private int y = 0;
	
	private int lastX = 0;
	private int lastY = 0;
	
	private int s = 0;
	private double v;
	private double u;
	private int horDirection;
	private double scale;
	
	private int alive;

	public Ball (int x, int y, int ballSize, int initialVelocity, int horDirection, double scale) {
		this.scale = scale;

		// Initialize all of the variables
		this.x = x;
		this.y = y;
		
		this.ballSize = ballSize;
		
		
		if (ballSize == 1) {
			ballPxlSize = 8;
			ballBounceSize = 30;
			ballPointValue = 400;
		} else if (ballSize == 2) {
			ballPxlSize = 16;
			ballBounceSize = 40;
			ballPointValue = 200;
		} else if (ballSize == 3) {
			ballPxlSize = 32;
			ballBounceSize = 50;
			ballPointValue = 100;
		} else if (ballSize == 4) {
			ballPxlSize = 41;
			ballBounceSize = 60;
			ballPointValue = 50;
		}

		ballPxlSize = scaleAndRound(ballPxlSize);
		ballBounceSize = halfScaleAndRound(ballBounceSize);
		
		this.horDirection = horDirection;

		u = initialVelocity;
		alive = 1;
	}
	
	
	public int getBallPxlSize () {
		return ballPxlSize;
	}
	
	public void update (int x, int y, double v, int s, int horDirection) {
		this.x = x;
		this.y = y;
		this.v = v;
		this.s = s;
		this.horDirection = horDirection;
	}
	
	public void update (int horMovement, int time, int width, int height) {
	
//		int ballPxlSize = scaleAndRound(ballPxlSize);
//		int ballBounceSize = halfScaleAndRound(ballBounceSize);
	
	
	
		lastX = x;
		lastY = y;
				
		// bounce the ball (give it an upward velocity)
		if (y >= height - ballPxlSize) {
			s = 0; // reset the distance
			u = -ballBounceSize; // change velocity depending on the size of the ball

		}
				
		// increase the horisontal components
		x = x + horMovement * horDirection;
		s++;

		// change the x direction if nessesary
		if (horDirection > 0 && x > width - ballPxlSize) horDirection = -horDirection;				
		if (horDirection < 0 && x < 0) horDirection = -horDirection;				

		//limit the x movement
		if (x < 0) 
			x = 0;
		if (x > width - ballPxlSize)
			x = width - ballPxlSize;
					
		
		double t = s * ((double) time / 1000);

		double a = BallLinked.a;


		// Calibrate the time to a frame rate of 1 every 200 milliseconds.
		// How this works:  The shorter the time, the more times the velocity is calculated, making it go higher.
		// To counter this, the gravity must increase also.  The balls were calibrated to 1/200, so anything smaller increases the gravity, and anthing larger decreases it
		// Note this doesn't fix the problem that the game seeds up with less wait.
		a = a * ((double) 200 / (double) time);

		v = u+a*t;
			
			
		// change the y coordinates
		y = y + (int) Math.round(v);
		
		u = v;
			
		//  if the ball is lower than the height, then set it to the lowest height
		if (y > height - ballPxlSize) y = height - ballPxlSize;
	}
	
	//Getter Methods
	public int getU () {
		return (int) u;
	}
	public int getX () {
		return x;
	}
	public int getLastX () {
		return lastX;
	}
	public int getY () {
		return y;
	}
	public int getLastY () {
		return lastY;
	}
	public double getV () {
		return v;
	}
	public int getS () {
		return s;
	}
	public int getHorDirection () {
		return horDirection;
	}	
	public int getBallSize () {
		return ballSize;
	}
	public int getBallPointValue() {
		return ballPointValue;
	}
	public String toString() {
		return "ball:" + x;
	}


	public int scaleAndRound (int number) {
		return (int) Math.round(number * scale);
	}

	public int halfScaleAndRound (int number) {
		if (scale < 2) {
			return number;
		} else {
			return number + scaleAndRound(8)-8;//(int) Math.round(number * scale /2);
		}
	}

}
	
